# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# This Source Code Form is "Incompatible With Secondary Licenses", as
# defined by the Mozilla Public License, v. 2.0.

package Bugzilla::Auth::Login::APIKey;

use 5.10.1;
use strict;
use warnings;

use base qw(Bugzilla::Auth::Login);

use Bugzilla::Constants;
use Bugzilla::User::APIKey;
use Bugzilla::Util;
use Bugzilla::Error;

use constant requires_persistence  => 0;
use constant requires_verification => 0;
use constant can_login             => 0;
use constant can_logout            => 0;

# This method is only available to web services. An API key can never
# be used to authenticate a Web request.
sub get_login_info {
  my ($self) = @_;
  my $params = Bugzilla->input_params;
  my ($user_id, $login_cookie);

  ## RED HAT EXTENSION START 1995385
  my $api_key_text;

  my $cgi         = Bugzilla->cgi;
  my $auth_header = $cgi->http('Authorization');

  if ($auth_header) {
    $auth_header = trim($auth_header);
    if ($auth_header =~ /^Bearer\s(.*)$/) {
      $api_key_text = trim($1);
    }
  }

  if ($params->{'Bugzilla_api_key'} && !$api_key_text) {
    $api_key_text = trim(delete $params->{'Bugzilla_api_key'});
  }
  ## RED HAT EXTENSION END 1995385

  if (!i_am_webservice() || !$api_key_text) {
    return {failure => AUTH_NODATA};
  }

  my $crypted_key
    = bz_crypt($api_key_text, Bugzilla->localconfig->{'site_wide_secret'},
    PASSWORD_DIGEST_ALGORITHM);

  my $api_key = Bugzilla::User::APIKey->new({name => $crypted_key});

  if (!$api_key or $api_key->api_key ne $crypted_key) {

    # The second part checks the correct capitalisation. Silly MySQL
    ThrowUserError("api_key_not_valid");
  }
  elsif ($api_key->banned) {
    Bugzilla::configure_log4perl_user($api_key->user);
    $api_key->increment_rejections();
    $api_key->update_last_used();
    ThrowUserError('api_key_banned');
  }
  elsif ($api_key->revoked) {
    Bugzilla::configure_log4perl_user($api_key->user);
    $api_key->increment_rejections();
    $api_key->update_last_used();
    ThrowUserError('api_key_revoked');
  }

  $api_key->update_last_used();

  return {user_id => $api_key->user_id};
}

1;
