#!/usr/bin/perl -w
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

use lib qw(. /var/www/html/bugzilla/);

use strict;
use warnings;

use Bugzilla;

main(@ARGV) if not caller();

sub main {
  my $all = shift();

  my $dbh = Bugzilla->dbh;

  $dbh->bz_start_transaction(1);

  if ($all) {
    $dbh->do("DELETE FROM logincookies");
  }
  else {
    # Get rid of any entry that hasn't been used in a day.
    if (Bugzilla->expire_cookies) {
      $dbh->do("DELETE FROM logincookies WHERE expires <= NOW() OR "
          . $dbh->sql_to_days('NOW()') . " - "
          . $dbh->sql_to_days('lastused')
          . " > 2");
    }
    else {
      $dbh->do("DELETE FROM logincookies WHERE "
          . $dbh->sql_to_days('NOW()') . " - "
          . $dbh->sql_to_days('lastused')
          . " > 2");
    }
  }
  $dbh->bz_commit_transaction();

  return;
}

1;
__END__

=head1 NAME

bz-expire-cookies.pl - remove expired cookies from db

=head1 SYNOPSIS

 cd /path/to/bugzilla && perl bz-expire-cookies.pl

 # cron example
 MAILTO="bugzilla-owner@redhat.com"
 # run 13 minutes past midnight every day
 0 13 * * * cd <your-bugzilla-directory>; ./bz-expire-cookies.pl 

=head1 DESCRIPTION

Asks bugzilla to remove all expired cookie tokens from the database.

=head1 ENVIRONMENT 

Needs to be in the root directory of the Bugzilla installation in order
to load the perl libs.

=head1 SEE ALSO

L<Bugzilla::Auth>

