-- Bug 1265486.  This script will rebuild the full text search indexes
-- Bug 2148334.  Use proceedure, requires PSQL 12.

CREATE INDEX IF NOT EXISTS comments_vect_idx ON bugs_fulltext USING gin(comments_vect);
CREATE INDEX IF NOT EXISTS comments_noprivate_vect_idx ON bugs_fulltext USING gin(comments_noprivate_vect);
CREATE INDEX IF NOT EXISTS short_desc_vect ON bugs_fulltext USING gin(short_desc_vect);

DROP FUNCTION IF EXISTS update_fulltext();

CREATE OR REPLACE PROCEDURE update_fulltext()
LANGUAGE plpgsql
AS $$
DECLARE
    rowdata RECORD;
    count integer := 0;
    loops_to_notice integer := 1000;
    t timestamptz := clock_timestamp();
    min_bug_id integer := 0;
BEGIN
    RAISE NOTICE 'Processing bugs_fulltext';
    FOR rowdata IN (SELECT bug_id FROM bugs WHERE bug_id >= min_bug_id ORDER BY bug_id)
    LOOP
        UPDATE bugs_fulltext 
           SET short_desc_vect = to_tsvector('english', quote_literal(escape_special_tokens(short_desc))),
               comments_noprivate_vect = to_tsvector('english', quote_literal(escape_special_tokens(comments_noprivate))),
               comments_vect = to_tsvector('english', quote_literal(escape_special_tokens(comments)))
         WHERE bug_id = rowdata.bug_id;
        COMMIT;

        count = count + 1;
        IF count % loops_to_notice = 0 THEN
            RAISE NOTICE 'count: %, last bug: %, time taken %', to_char(count, '9,999,999'), rowdata.bug_id, clock_timestamp() - t;
        END IF;
    END LOOP;

    RAISE NOTICE 'sum:   %, total time: %', to_char(count, '9,999,999'), clock_timestamp() - t;

END;
$$;

CALL update_fulltext();

DROP PROCEDURE update_fulltext();

VACUUM ANALYZE bugs_fulltext;
