#!/usr/bin/perl
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

use strict;
use warnings;
use 5.10.1;

use File::Slurp qw( edit_file prepend_file);
use utf8::all;

=pod

Script to update change log and release notes.

Before you run this file you have to:

 git checkout $branch
 git pull
 vim Bugzilla/Constants.pm
   change BUGZILLA_VERSION
   save

Then you run the git commands it prints out.

=cut

#my $curr_tag = qx/git describe --tags --abbrev=0 --match=Release*/;
my $curr_tag = qx/git for-each-ref --sort=creatordate --format '%(tag)' | grep -E 'Release.*rh.*' | tail -n1/;
chomp($curr_tag);
my $version
  = qx{grep "constant BUGZILLA_VERSION" Bugzilla/Constants.pm | cut -f2 -d'"'};
chomp($version);

# This assumes the release is set in Constants.pm
# It wouldn't be to hard to bump it here and update that file ...
$version =~ s/\.([^.]*)$/-$1/;

my $next_tag = "Release-$version";
my $cmd = qq/git log $curr_tag..HEAD --no-merges --format=" - %s"  | sort -u /;
my $log = qx/$cmd/;

my $name = qx/git config --get user.name/;
chomp($name);

my $email = qx/git config --get user.email/;
chomp($email);

$cmd
  = qq{git log $curr_tag..HEAD --no-merges --format="    <li>\%s</li>" | grep -E 'li>Bug [0-9]+' | sort -u };
my $html_log = qx/$cmd/;
$html_log =~ s{Bug (\d+)}{<a href="https://bugzilla.redhat.com/$1">Bug $1</a>}g;
$html_log =~ s{a>\s*-\s*}{a> }g;

# This template just contains the HTML generated above and is included in the release notes
# by adding this to template/en/default/pages/release-notes.html.tmpl
#  <h2 id="point">Updates in this 5.0.x Release</h2>
#
#+ [% Hook.process('updates_top') %]

prepend_file(
  'extensions/RedHat/template/en/default/hook/pages/release-notes-updates_top.html.tmpl',
  "<h3 id='$version'>$version</h3>\n<ul>\n$html_log\n</ul>\n\n"
);

say <<EOM;
now run:

git commit -a -m "Release Version $version"
git tag -a "Release-$version" -m "Release Version $version"
git push --follow-tags
git checkout devel
git rebase master
git push
bz-admin jenkins-build --tag bugzilla-$version

EOM

