# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

package Bugzilla::Extension::RedHat::DocTypeText;

use strict;
use warnings;
use 5.10.1;

use base qw(Bugzilla::Object);

use Bugzilla::Constants;
use Bugzilla::Error;
use Bugzilla::Util qw(diff_arrays trick_taint);

use File::Basename;

use constant DB_COLUMNS => qw(
  id
  cf_doc_type_id
  value
);

use constant DB_TABLE   => 'doc_type_text';
use constant LIST_ORDER => 'cf_doc_type_id';
use constant ID_FIELD   => 'id';
use constant NAME_FIELD => 'cf_doc_type_id';

use constant REQUIRED_CREATE_FIELDS => qw(
  cf_doc_type_id
  value
);

use constant UPDATE_COLUMNS => qw(
  cf_doc_type_id
  value
);

use constant VALIDATORS =>
  {cf_doc_type_id => \&_check_cf_doc_type_id, value => \&_check_value,};

###############################
####     Constructors     #####
###############################

sub new {
  my $invocant = shift;
  my $class    = ref($invocant) || $invocant;
  my $param    = shift;

  if (ref($param) eq 'HASH' and defined($param->{name})) {
    my $value = Bugzilla->dbh->quote($param->{name});
    my $c     = "(SELECT id FROM cf_doc_type WHERE value = $value)";

    $param = {condition => "cf_doc_type_id IN $c", values => []};
  }

  my $self = $class->SUPER::new($param);

  return $self;
}

sub set_cf_doc_type_id { $_[0]->set('cf_doc_type_id', $_[1]); return; }
sub set_value          { $_[0]->set('value',          $_[1]); return; }

###############################
####      Accessors      ######
###############################

sub id             { return $_[0]->{'id'}; }
sub cf_doc_type_id { return $_[0]->{'cf_doc_type_id'}; }
sub value          { return $_[0]->{'value'}; }

###############################
####     Validators        ####
###############################

sub _check_cf_doc_type_id {
  my ($invocant, $cf_doc_type_id) = @_;
  if (!defined $cf_doc_type_id || $cf_doc_type_id eq '') {
    ThrowUserError("required_parameter_missing",
      {func => 'check_cf_doc_type_id', param => 'cf_doc_type_id'});
  }
  return $cf_doc_type_id;
}

sub _check_value {
  my ($invocant, $value) = @_;
  if (!defined $value || $value eq '') {
    ThrowUserError("required_parameter_missing",
      {func => 'check_value', param => 'value'});
  }
  return $value;
}

1;

__END__

