/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */

const bz_channel = new BroadcastChannel('bugzilla_channel');

bz_channel.onmessage = function (e) {
  if (e.data === 'login') {
    checkSessionExpires();
    close_relogin();
  }
};

function show_relogin() {
  $("#relogger").width('490px');
  $("#relogger").height('230px');
  $("#relogger").removeClass("bz_default_hidden");
}

function close_relogin() {
  $("#relogger").addClass("bz_default_hidden");
  $('#relogin_frame').off('load');
  $("#relogin_frame").attr("src", "/newsession.cgi");
}

var keys = {
  ENTER: 13,
  ESC: 27,
  F1: 112,
  F12: 123,
  LEFT: 37,
  RIGHT: 39,
  TAB: 9
};

alertify.defaults.notifier.position = 'top-right';
alertify.defaults.transitionOff = true;
alertify.defaults.modal = false;

if (!alertify.editCommentBox) {
  alertify.dialog('editCommentBox', function factory() {
    var notes = $('<p><b>Note:</b> To make the comment edit history hidden, you need to mail <a href="mailto:bugzilla-owner@redhat.com"> bugzilla-owner@redhat.com</a> with the bug and comment details.</p>');
    var input = $('<textarea rows="15" cols="50"></textarea>');
    var sec_label = $('<label>Lock history: </label>');
    var sec_input = $('<input type="checkbox"></input>');
    var minor_label = $('<label>This is a minor update (do not send email): </label>');
    var minor_input = $('<input type="checkbox"></input>');
    var space = document.createTextNode(" ");;
    var comment;
    var c_count;
    return {
      main: function (text_elem, comment_id, comment_count, message, allow_secure) {
        comment = text_elem;
        this.comment_id = comment_id;
        c_count = comment_count;
        this.message = message;
        this.set('title', 'Edit comment ' + comment_count);
        this.secure = allow_secure;
      },
      setup: function () {
        return {
          buttons: [{
              text: alertify.defaults.glossary.ok,
              className: alertify.defaults.theme.ok,
            },
            {
              text: alertify.defaults.glossary.cancel,
              key: keys.ESC,
              invokeOnClose: true,
              className: alertify.defaults.theme.cancel,
            }
          ],
          focus: {
            element: null,
            select: false
          },
        };
      },
      build: function () {
        input.className = alertify.defaults.theme.input;
        this.elements.content.appendChild(input[0]);
        this.elements.content.appendChild(minor_label[0]);
        this.elements.content.appendChild(minor_input[0]);
        this.elements.content.appendChild(space);
      },
      prepare: function () {
        $(input).val(this.message);
        if (this.secure) {
          this.elements.content.appendChild(sec_label[0]);
          this.elements.content.appendChild(sec_input[0]);
        } else {
          this.elements.content.appendChild(notes[0]);
        }
      },
      callback: function (closeEvent) {
        var returnValue;
        switch (closeEvent.index) {
          case 0:
            var new_val = input[0].value;
            if (new_val !== this.message) {
              $(input)._onAjaxSend();
              // RPC to update message
              var rpc = new Rpc('RedHat', 'edit_comment', {
                  comment: this.comment_id,
                  text: new_val,
                  render: 1,
                  security: $(sec_input).prop("checked") ? 1 : 0,
                  minor_update: $(minor_input).prop("checked") ? 1 : 0,
                })
                .complete(function () {
                  $(input)._onAjaxComplete();
                })
                .fail(function (error) {
                  warn(error);
                })
                .done(function (res) {
                  old_ts = res['delta_ts'];
                  old_time = new Date(old_ts).getTime()
                  $('[name="delta_ts"]').val(old_ts);
                  $(comment).html(res['html']);
                  $('#comment_history_' + c_count).removeClass('bz_default_hidden');
                  $('#comment_activity_count_' + c_count).html(res['activity_count']);
                });
            } else {
              alertify.warning("No changes detected!");
            }
            break;
          case 1:
            // NOP
            break;
        }
        $(input).remove();
      }
    };
  }, true);
}

function edit_comment(comment_id, comment_count, allow_secure) {
  var text_elem = document.getElementById('comment_text_' + comment_count);
  var text = getText(text_elem);
  alertify.editCommentBox(text_elem, comment_id, comment_count, text, allow_secure).resizeTo('80%', '70%');
}

function invite_manager(manager) {
  $('#thelink')._onAjaxSend();
  // RPC to update message
  var rpc = new Rpc('GroupRequest', 'invite_manager', {
      manager: manager
    })
    .complete(function () {
      $('#thelink')._onAjaxComplete();
    })
    .fail(function (error) {
      warn(error);
    })
    .done(function (res) {
      alertify.success(res);
    });
}

// Cache of marked up user names 
var user_cache = {};

// Get this from Constants.pm
var FIELD_TYPE_HIGHEST_PLUS_ONE = 12;

var FieldType = {
  UNKNOWN: 0,
  STRING: 1,
  SELECT: 2,
  MULTI: 3,
  TEXT: 4,
  DATETIME: 5,
  BUGID: 6,
  URL: 7,
  KEYWORDS: 8,
  DATE: 9,
  INT: 10,
  ONE_SELECT: 11,
  USER: FIELD_TYPE_HIGHEST_PLUS_ONE,
  BOOLEAN: FIELD_TYPE_HIGHEST_PLUS_ONE + 1
};

/*
  HTML format a field from a Data=Tables row
*/

function format_field(data, row, field, nomarkup) {

  var nomarkup = (typeof nomarkup !== 'undefined') ? nomarkup : false;

  var fld_name = field;
  if (typeof (BB_FIELDS[field]) === 'undefined' &&
    typeof (field_map[field]) !== 'undefined'
  ) {
    fld_name = field_map[field];
  }

  if (typeof (BB_FIELDS[fld_name]) !== 'undefined') {
    if (BB_FIELDS[fld_name].type == FieldType.USER) {
      return user_link(data, row, fld_name, nomarkup);
    } else if (BB_FIELDS[fld_name].type == FieldType.BUGID) {
      var str = '';
      if (BB_FIELDS[fld_name].multivalue) {
        for (var i in data) {
          str = str + bug_link(data[i], row, nomarkup);
        }
      } else {
        str = bug_link(data, row, nomarkup);
      }
      return str;
    } else if (BB_FIELDS[fld_name].type == FieldType.DATETIME) {
      // Convert times to users local TZ
      var str = data;

      if (str !== null &&
        (typeof (BB_CONFIG['user']) !== 'undefined') &&
        (typeof (BB_CONFIG['user']['tz']) !== 'undefined') &&
        (BB_CONFIG['user']['tz'] !== 'UTC')
      ) {
        var localTime = new Date(data);
        str = formatDateTime(localTime);
      }

      return str;
    }
  }

  if (field == 'sub_components') {
    return format_sub_component(data, row, nomarkup);
  } else if (field == 'component') {
    return format_component(data, row, nomarkup);
  } else if (field == 'product') {
    return format_product(data, row, nomarkup);
  } else if (field == 'flagtypes.name') {
    return format_flags(data, row, false);
  } else if (field == 'flags') {
    return format_flags(data, row, true);
  } else if (field == 'ext_bz_count') {
    return format_ext_bz_count(data, row);
  } else if (field == 'ext_bz_list') {
    return format_ext_bz_list(data, row, nomarkup);
  } else if (field == 'longdescs.count') {
    return format_comment_count(data, row);
  } else if (field == 'summary') {
    return summary_link(data, row, nomarkup);
  }

  var str = '';
  if (typeof (data) !== 'undefined') {
    if (Array.isArray(data)) {
      str = escapeHtml(data.join(' '));
    } else {
      str = escapeHtml(data);
    }
  }

  return str;
}

function zeroPadNum(theNumber) {
  if (theNumber < 10) {
    theNumber = "0" + theNumber;
  }
  return theNumber;
}

// ISO time but more readable ... and local.
function formatDateTime(td) {
  return td.getFullYear() + '-' +
    zeroPadNum(td.getMonth() + 1) + '-' +
    zeroPadNum(td.getDate()) + ' ' +
    zeroPadNum(td.getHours()) + ':' +
    zeroPadNum(td.getMinutes()) + ':' +
    zeroPadNum(td.getSeconds());
}

function format_comment_count(data, row) {
  var str = 0;
  if (typeof (row['longdescs.count']) !== 'undefined') {
    str = row['longdescs.count'];
  }
  return str;
}

function format_ext_bz_list(data, row, nomarkup) {
  var str = '';
  if (typeof (row['external_bugs']) !== 'undefined') {
    for (var i = 0; i < row['external_bugs'].length; i++) {
      str = str + format_et_link(row['external_bugs'][i], nomarkup);
      if (nomarkup) {
        str += ' ';
      }
    }
  }
  return str;
}

function format_ext_bz_count(data, row) {
  var str = 0;
  if (typeof (row['external_bugs']) !== 'undefined') {
    str = row['external_bugs'].length;
  }
  return str;
}

function format_sub_component(data, row, nomarkup) {
  if (typeof (data[row.component[0]]) === 'undefined') {
    return '';
  }

  if (nomarkup) {
    return data[row.component[0]].join(' ');
  }

  return '<span class="nowrap">' + data[row.component[0]].join('</span><i class="fas fa-arrow-right"></i><span class="nowrap">') + '</span>';
}

function format_component(data, row, nomarkup) {
  if (typeof (data[0]) === 'undefined') {
    return '';
  }

  if (nomarkup || typeof (row.product) === 'undefined') {
    return escapeHtml(data[0]);
  }

  return '<span class="nowrap">' + escapeHtml(data[0]) + '<a href="enter_bug.cgi?component=' + escapeHtml(data[0]) + '&product=' + escapeHtml(row.product) + '" title="Create a new bug for this component">&nbsp;<i class="fas fa-plus-circle"></i></a></span>';
}

function format_product(data, row, nomarkup) {
  if (typeof (data) === 'undefined') {
    return '';
  }

  if (nomarkup) {
    return escapeHtml(data);
  }

  return '<span class="nowrap"> ' + escapeHtml(data) + '</span><a href="enter_bug.cgi?product=' + escapeHtml(data) + '" title="Create a new bug for this product">&nbsp;<i class="fas fa-plus-circle"></i></a></span>';
}

function format_flags(data, row, full) {
  var str = '';
  if (typeof (row['flags']) !== 'undefined') {
    for (var i = 0; i < row['flags'].length; i++) {
      str = str + row['flags'][i].name + row['flags'][i].status;
      if (full && typeof (row['flags'][i]['requestee']) !== 'undefined') {
        str += row['flags'][i]['requestee'];
      }
      str += " ";
    }
  }
  return str;
}

/*
  Format a Bug link for a Data-tables row
*/

function bug_link(bug_id, row, nomarkup) {
  if (typeof bug_id === 'undefined' || bug_id === '' || bug_id === 'null') {
    return '';
  }

  if (nomarkup) {
    return bug_id;
  }

  var link = '<a class="bz_bug_link ';
  if (typeof row.bug_classes !== 'undefined') {
    link += row.bug_classes;
  }
  link += '"';

  if (typeof row.summary !== 'undefined') {
    link += " title='" + escapeHtml(row.summary) + "'";
  }

  link += ' href="show_bug.cgi?id=' + bug_id + '"';
  link += '>';

  if (typeof row.data_category !== 'undefined' && row.data_category !== 'Public') {
    link += '<span class="bz_keyword bz_data_category">' + row.data_category + '</span>';
  }

  link += bug_id;
  link += '</a>';

  return (link);
}

function summary_link(summary, row, nomarkup) {
  if (typeof summary === 'undefined' || summary === '' || summary === 'null') {
    return '';
  }

  var str = escapeHtml(summary);

  if (nomarkup) {
    return str;
  }

  var link = '<a class="bz_bug_link ';
  if (typeof row.bug_classes !== 'undefined') {
    link += row.bug_classes;
  }

  link += '"' + " title='" + str + "'";
  link += ' href="show_bug.cgi?id=' + row.id + '"';
  link += '>';

  if (typeof row.data_category !== 'undefined' && row.data_category !== 'Public') {
    link += '<span class="bz_keyword bz_data_category">' + row.data_category + '</span>';
  }

  link += str;
  link += '</a>';

  return (link);
}

function team_link(team_name, row, nomarkup) {

  var str = escapeHtml(team_name);

  if (nomarkup) {
    return str;
  }

  var link = '<a href="page.cgi?id=agiletools/team/show.html&team_id=';
  link += row.id;
  link += '">';
  link += str;
  link += '</a>';

  return (link);
}

/*
  Format a User name for a Data-tables row
*/

function user_link(login, row, field, nomarkup) {
  if (typeof login === 'undefined' || login === '' || login === 'null') {
    return '';
  }

  if (nomarkup || typeof (BB_CONFIG.user.id) === 'undefined') {
    return login;
  }

  if (typeof (user_cache[login]) !== 'undefined') {
    return user_cache[login];
  }

  var name = login;
  var id = '';
  var span = login;

  if (typeof (row[field + "_detail"]) !== 'undefined') {
    if (row[field + "_detail"].real_name !== '') {
      name = escapeHtml(row[field + "_detail"].real_name);
      id = row[field + "_detail"].id;
    }
  }

  var link = '<ul class="related_actions users message">';

  link += `  
    <li>
      <a href="page.cgi?id=user_activity.html&amp;action=run&amp;who=${login}"
            title="View Bug Activity for '${login}'">
            <i class="fas fa-bug"></i>
      </a>
    </li>
  `;

  // BUGBUG need to check this for each user?
  if (BB_CONFIG.user.groups.includes('editusers')) { // || user.can_bless) {
    link += `  
      <li>
        <a href="editusers.cgi?action=edit&amp;userid=${id}"
            title="Edit User '${login}'">
            <i class="fas fa-user"></i>
        </a>
      </li>
    `;
  }
  if (BB_CONFIG.user.groups.includes('editusers')) {
    link += `  
      <li>
        <a href="editusers.cgi?action=activity&amp;userid=${id}"
            title="View Account History for '${login}'">
            <i class="fas fa-history"></i>
        </a>
      </li>
      <li>
        <a href="page.cgi?id=user_admin_activity.html&amp;action=run&amp;who=${login}"
              title="View Admin Activity for '${login}'">
              <i class="fas fa-user-cog"></i>
        </a>
      </li>
      <li>
        <a href="page.cgi?id=redhat/admin_user_sessions.html&amp;who=${login}"
              title="Admin sessions '${login}'">
              <i class="fas fa-tools"></i>
        </a>
      </li>
      <li>
        <a href="userprefs.cgi?other_user=${login}"
              title="Edit preferences for '${login}'">
              <i class="fas fa-user-tie"></i>
        </a>
      </li>

      <li>
        <a href="page.cgi?id=redhat/admin_user_access.html&amp;who=${login}"
              title="Admin access for '${login}'">
              <i class="fas fa-power-off"></i>
        </a>
      </li>
    `;
  }

  link += '</ul>';

  var classes = 'vcard';
  if (row[field + "_detail"].insider) {
    classes = classes + ' redhat_user';
  }

  if (row[field + "_detail"].partner) {
    classes = classes + ' partner_user';
  }

  if (!row[field + "_detail"].active) {
    classes = classes + ' bz_inactive';
  }

  span = `
    <span class="${classes}">
      <a class="email" href="mailto:${login}">
        <span class="fn">${name}</span>
      </a>
      <span class="user_info">
        <i class="fas fa-tools"></i>
        ${link}
      </span>
    </span>
  `;

  user_cache[login] = span;

  return (span);
}

function format_et_link(tracker, nomarkup) {
  var css_class = '';

  var et = tracker['type']['description'];

  var url = tracker['type']['full_url'];
  url = url.replace('%id%', escapeHtml(tracker['ext_bz_bug_id']));

  if (nomarkup) {
    return url;
  }

  if (et == 'Red Hat Errata Tool') {
    css_class = "et_et";
  } else if (et == 'Red Hat Product Errata') {
    css_class = "et_access";
  } else if (et.match("Red Hat Customer Portal")) {
    css_class = "et_portal";
  } else if (et.match("TCMS")) {
    css_class = "et_tcms";
  } else if (et.match("Polarion")) {
    css_class = "et_polarion";
  } else if (et == 'Red Hat Bugzilla') {
    css_class = "et_rhbz";
  } else if (et == 'Mozilla Foundation') {
    css_class = "et_bmo";
  } else if (et.match("Red Hat Knowledge Base")) {
    css_class = "et_rhkb";
  } else if (et.match("CPAN")) {
    css_class = "et_cpan";
  } else if (et.match("^IBM")) {
    css_class = "et_ibm";
  } else if (et.match(/gerrit/i)) {
    css_class = "et_gerrit";
  } else if (et.match("Red Hat Issue Tracker")) {
    css_class = "et_rhit";
  } else if (et == "Red Hat ServiceNow") {
    css_class = "et_rhsn";
  }

  var title = "[" + et + "]";
  if (tracker['ext_description'] == "None") {
    title += " No Summary";
  } else {
    title += " " + escapeHtml(tracker['ext_description']);
  }

  var link = '<a class="et_button ' + css_class + '"';

  link += ' title="' + title + '"';
  link += ' href="' + url + '"';
  link += '>' + escapeHtml(tracker['ext_bz_bug_id']) + '</a> ';

  return link;
}

// field: jquery object
// field_type: from FieldType object
// values: data source for options
function createInput(id, field_name) {
  var element;
  var field_type = BB_FIELDS[field_name].type;

  if (field_type == FieldType.SELECT ||
    field_type == FieldType.MULTI ||
    field_type == FieldType.ONE_SELECT ||
    (field_type == FieldType.KEYWORDS && field_name == "keywords")
  ) {
    element = $("<select></select>");
  } else if (field_type == FieldType.TEXT) {
    element = $("<textarea></textarea>");
  } else if (field_type == FieldType.BOOLEAN) {
    element = $("<input type='checkbox' />");
  } else {
    element = $("<input></input>");
    element.addClass('text_input field_value');
    element.addClass(alertify.defaults.theme.input);
  }
  element.attr("name", id);
  element.attr("id", id);

  if (element.is('select')) {
    if (field_type == FieldType.MULTI ||
      (BB_FIELDS[field_name].multivalue == 1)
    ) {
      element.attr('multiple', 'multiple');
    }
    // BUGBUG if it's a single value can we set this??
    var values = [];

    if (typeof (BB_FIELDS[field_name].values) !== 'undefined') {
      values = BB_FIELDS[field_name].values;
    }
    _setSelectOptions(element, values); //, field.text());
  } else if (field_type == FieldType.BOOLEAN) {
    //    var value = field.html();
    //    element.prop("checked", value === "1");
  } else {
    //    var value = field.html();
    // BUGBUG missing this!
    //        if (value == undefined) value = this.defaultValue(field);
    //element.val('--do_not_change--');
  }

  if (field_type == FieldType.USER) {
    element.userautocomplete({
      multiple: BB_FIELDS[field_name].multivalue == 1
    });
  }

  return (element);
}

function _setSelectOptions(element, values, blank, currentValue) {
  if (!element.is('select')) return;

  var blank = (typeof blank !== 'undefined') ? blank : true;

  element.empty();
  if (blank) {
    var blank_option = $('<option value=""></option>')
    element.append(blank_option);
  }

  if (!$.isArray(currentValue)) currentValue = [currentValue];
  $.each(values, function (value) {
    var val = values[value].name;
    var option = $('<option>' + val + '</option>')
      .attr('value', val);

    if (typeof values[value].title !== 'undefined') {
      option.attr('title', values[value].title);
    }

    if ((currentValue.length == 0 && val == defaultValue) ||
      (currentValue.indexOf(val) != -1)) {
      option.attr('selected', 'selected');
      element.prepend(option);
    } else {
      if (name != 'product' ||
        BB_CONFIG.user.enterable_products.indexOf(val) != -1) {
        element.append(option);
      }
    }
  });
};

var bug_ids = [];

function selecterize(element) {

  $(element).selectize({
    plugins: ['remove_button',
      'minimum_search_length',
      'extra_keys_control',
      'related_fields',
      'load_from_js',
      'auto_position'
    ],
    maxOptions: 'nolimit',
    minimum_search_length: 2,
    render: {
      option: function (item, escape) {
        var classes = "option";
        var selected = this.items.indexOf(item.value) !== -1;
        if (item.disabled) {
          classes = classes + " bz_default_hidden";
        }

        return '<div class="' + classes + '" title="' +
          escape(item.title) + '" id="' + escape(item.id) +
          '" data-value="' + escape(item.value) + '">' +
          '<span>' + escape(item.text) + '</span>' +
          '</div>';
      }
    },
  });

}

function set_edit_product() {
  var $select = $("#product").selectize();
  var $selectize = $select[0].selectize;
  var prod = Object.keys(selected_products)[0];
  $selectize.setValue(prod);
  $('#changed_product').prop("checked", false);
}

function submit_bug_edits() {
  $('.ajs-header')._onAjaxSend();

  var num_changes = 0;
  var changes = {};
  changes["ids"] = bug_ids;

  $('input[type="checkbox"]:checked').each(function (index, field) {
    var id = $(field).attr('id');
    // changed_...
    if (id.match("^changed_")) {
      var field_name = id.substring(8);
      var esc_name = field_name.replace(/\./g, '\\.');
      if (field_name === 'comment') {
        var comment = {};
        comment['body'] = $('#' + esc_name).val();
        if ($('#newcommentprivacy').prop("checked")) {
          comment['is_private'] = 1;
        }

        changes[field_name] = comment;
      } else if (field_name === 'rh_sub_components') {
        var comp = $('#component').get_selected_item();
        var sub_comp = $('#' + field_name).get_selected_item();
        changes['sub_components'] = {};
        changes['sub_components'][comp] = [sub_comp];
      } else if ($('#ars_' + field_name).length) {
        var action = $('#ars_' + field_name).val();
        var items;

        if ($('#' + esc_name).hasClass('selectized')) {
          items = $('#' + esc_name).get_selected_items();
        } else {
          items = $('#' + esc_name).val().split(/,\s*/);
        }

        // remove last item if it's blank ...
        if (items.length > 0) {
          if (items[items.length - 1] == "") {
            items.pop();
          }
        }

        var obj = {};
        obj[action] = items;
        changes[field_name] = obj;
      } else if (field_name.match("^flag_type-")) {

        var indx = $('#' + field_name).attr('data-index-number');
        var fid = BB_FIELDS['flagtypes.name'].values[indx].id;
        var obj = {};
        obj.type_id = fid;
        obj.status = $('#' + field_name).get_selected_item();
        if (BB_FIELDS['flagtypes.name'].values[indx].is_requesteeble) {
          obj.requestee = $('#flag_requestee-' + fid).get_selected_item();
        }

        if (typeof (changes['flags']) === 'undefined') {
          changes['flags'] = [];
        }
        changes['flags'].push(obj);
      } else {
        if ($('#' + esc_name).prop("type") == "checkbox") {
          changes[field_name] = $('#' + esc_name).prop("checked");
        } else {
          changes[field_name] = $('#' + esc_name).val();
        }
      }
      num_changes++;
    }
  });

  if ($('#set_default_pool').prop("checked")) {
    num_changes++;
    changes['set_default_pool'] = 1;
  }

  if ($('#set_dupe_of').prop("checked")) {
    num_changes++;
    changes['dupe_of'] = $('#dupe_of').val();
  }

  if (num_changes == 0) {
    alertify.error("No changes detected!");
    return;
  }

  if (bug_ids.length == 0) {
    alertify.error("No bugs selected!");
    return;
  }

  if ($('#minor_update').prop("checked")) {
    changes['minor_update'] = 1;
  }

  // RPC to update message
  var rpc = new Rpc('Bug', 'update', changes)
    .complete(function () {
      $('.ajs-header')._onAjaxComplete();
    })
    .fail(function (error) {
      warn(error);
    })
    .done(function (res) {
      alertify.warning(JSON.stringify(res, null, 2));
      $('#bz_buglist').DataTable().rows().deselect();
      $('#bz_buglist').DataTable().clearPipeline().draw();
    });
}

var first_show = true;
var uncheck = false;
var cals = [];

if (!alertify.editBugsBox) {
  alertify.dialog('editBugsBox', function factory() {
    return {
      main: function (num_bugs) {
        this.set('title', 'Bulk Edit ' + num_bugs + " bug[s]");
        $(this.elements.root).addClass('editBugsBox');
      },
      setup: function () {
        return {
          buttons: [{
              text: 'Submit',
              className: alertify.defaults.theme.ok,
            },
            {
              text: alertify.defaults.glossary.cancel,
              key: keys.ESC,
              invokeOnClose: true,
              className: alertify.defaults.theme.cancel,
            }
          ],
          focus: {
            element: null,
            select: false
          },
          options: {
            modal: true,
            autoReset: false
            /*            onclose: function () {
                          //alert("in onclose function!");
                        },
                        invokeOnCloseOff: true
            */
          }
        };
      },
      hooks: {
        // triggered when the dialog is closed, this is separate from user defined onclose
        //onclose: function () {
        //alert("in onclose hook!");
        //},
        onshow: function () {
          if (!first_show && Object.keys(selected_products).length == 1) {
            set_edit_product();
          }
        },
      },
      build: function () {
        //input.className = alertify.defaults.theme.input;
        var bug_form = $('<div class="bug_form_block"></div>');
        var top_block = $('<div class="top_block"></div>');
        var product_block = $('<details><summary class="message">Product Specific Fields (Selecting a product will hide and reveal fields and values)</summary></details>');
        var other_block = $('<details><summary class="message">Other Fields</h2><summary></details>');
        var space = document.createTextNode(" ");;

        var minor_cb = $('<div class="field_block"><label for="minor_update">This is a minor update (do not send email)</label> <input type="checkbox" id="minor_update" name="minor_update" value="1"></div>');
        top_block.append($(minor_cb));

        var reset_pool_cb = $('<div class="field_block"><label for="set_default_pool">Reset Pool to default</label><input type="checkbox" name="set_default_pool" value="1" id="set_default_pool"></div>');
        top_block.append($(reset_pool_cb));

        var needinfo_roles = $(`
          <div class="field_block">
            <input type="checkbox" id="changed_needinfo_roles"></input>
            <label for="needinfo_roles"
              title="Needinfo users based on their role on a bug"
            >Needinfo Roles</label>
          </div>
        `);

        var needinfo_select = $("<select></select>");
        needinfo_select.attr("name", 'needinfo_roles');
        needinfo_select.attr("id", 'needinfo_roles');
        needinfo_select.attr("multiple", 'multiple');

        needinfo_select.append($('<option value=""></option>'));
        needinfo_select.append($('<option value="assigned_to">Assignee</option>'));
        needinfo_select.append($('<option value="reporter">Reporter</option>'));
        needinfo_select.append($('<option value="qa_contact">QA Contact</option>'));
        needinfo_select.append($('<option value="docs_contact">Docs Contact</option>'));

        $(needinfo_select).on('change', function () {
          $("#changed_needinfo_roles").prop("checked", true);
        });

        needinfo_roles.append(needinfo_select);
        other_block.append(needinfo_roles);
        selecterize(needinfo_select);

        for (var i in BB_FIELD_ORDER) {
          var field_name = BB_FIELD_ORDER[i];
          if (typeof (BB_FIELDS[field_name]) !== 'undefined' &&
            field_name !== "summary" && field_name !== "alias" && field_name !== "flagtypes.name"
          ) {

            var div = $('<div class="field_block"></div>');
            var element = createInput(field_name, field_name);

            var label = $('<label for="' + field_name + '">' +
              BB_FIELDS[field_name].display_name + '</label>');

            if (BB_FIELDS[field_name].long_desc.length > 110) {
              $(label).attr('title', 'Click the information icon to the right to see the description');
            } else {
              $(label).attr('title', BB_FIELDS[field_name].long_desc);
            }


            var cb = $('<input type="checkbox"></input>');
            cb.attr("name", 'changed_' + field_name);
            cb.attr("id", 'changed_' + field_name);
            cb.attr("title", 'If this checkbox is not set then the ' + BB_FIELDS[field_name].display_name + ' field will not be changed regardless of the value in it.');

            if (field_name === "classification") {
              $(div).addClass("bz_default_hidden");
            }

            $(div).append($(cb));
            $(div).append($(space));
            $(div).append($(label));

            if (BB_FIELDS[field_name].long_desc.length > 110) {
              // This is allowed to have HTML
              //var txt = $("<div>" + BB_FIELDS[field_name].long_desc + "</div>");
              var info = $('<i class="fas fa-info-circle pop-text" onclick="alertify.alert(BB_FIELDS[\'' +
                field_name + '\'].display_name, BB_FIELDS[\'' +
                field_name + '\'].long_desc)" title="Click to see full description"></i>'
              );
              $(div).append($(info));
            }

            if (field_name === 'comment') {
              var private_cb = $('<input type="checkbox" name="comment_is_private" value="1" id="newcommentprivacy" onclick="updateCommentTagControl(this, \'comment\')"><label for="newcommentprivacy">Is private</label>');
              $(div).append($(private_cb));
            }

            $(div).append($(element));

            if (BB_FIELDS[field_name].type == FieldType.MULTI ||
              (BB_FIELDS[field_name].multivalue == 1)
            ) {
              var ars = $("<select class='short'></select>");
              ars.attr("name", 'ars_' + field_name);
              ars.attr("id", 'ars_' + field_name);

              // CC is multi but does not support set, there may be others
              var opts = [{
                "name": 'add'
              }, {
                "name": 'remove'
              }];

              if (field_name !== "cc") {
                opts.push({
                  "name": 'set'
                });
              }

              _setSelectOptions(ars, opts, false);

              $(div).append($(ars));
              $(div).addClass('short');
              selecterize(ars);
            } else if (
              (BB_FIELDS[field_name].type == FieldType.DATE) ||
              (BB_FIELDS[field_name].type == FieldType.DATETIME)
            ) {
              $(element).datepicker({
                dateFormat: "yy-mm-dd",
              });
            }

            if ((BB_FIELDS[field_name].visibility_field == 'product') ||
              (BB_FIELDS[field_name].visibility_field == 'classification') ||
              (BB_FIELDS[field_name].visibility_field == 'component') ||
              (BB_FIELDS[field_name].value_field == 'classification') ||
              (BB_FIELDS[field_name].value_field == 'product') ||
              (BB_FIELDS[field_name].value_field == 'component')
            ) {

              product_block.append($(div));
              $(div).addClass('bz_default_hidden');
            } else if (BB_FIELDS[field_name].sortkey > 99) {
              other_block.append($(div));
            } else {
              top_block.append($(div));
            }

            if (BB_FIELDS[field_name].visibility_field != null &&
              BB_FIELDS[field_name].visibility_field != ""
            ) {
              $(div).addClass('bz_default_hidden');
            }

            if (element.is('select')) {
              if (field_name !== 'product' &&
                field_name !== 'component' &&
                field_name !== 'rh_sub_components'
              ) {
                selecterize(element);
              }
            }

            $(element).on('change', function () {
              $('#changed_' + $(this).prop('id')).prop("checked", true);
              if ($(this).attr('id') == 'product') {
                update_classification();
                show_hide_flags();
              }
            });
          }
        }

        if (typeof (BB_FIELDS['flagtypes.name']) !== 'undefined') {
          if (typeof (BB_FIELDS['flagtypes.name'].values) !== 'undefined') {
            var flag_block = $('<div class="flag_block"><div class="message">Flags</div></div>');

            for (var indx in BB_FIELDS['flagtypes.name'].values) {
              var div = $('<div class="field_block flag short"></div>');

              var flag = BB_FIELDS['flagtypes.name'].values[indx];
              var id = 'flag_type-' + flag.id;

              var element = $(`<select class="flag_select">
              </select>`);
              $(element).append($('<option value="X"></option>'));
              $(element).append($('<option value="?">?</option>'));
              $(element).append($('<option value="+">+</option>'));
              $(element).append($('<option value="-">-</option>'));

              element.attr("name", id);
              element.attr("id", id);
              element.attr("data-index-number", indx);

              var cb = $('<input type="checkbox"></input>');
              cb.attr("name", 'changed_' + id);
              cb.attr("id", 'changed_' + id);

              $(element).on('change', function () {
                $("#changed_" + $(this).attr('id')).prop("checked", true);
              });

              var label = $('<label for="' + id + '">' +
                flag.name + '</label>');

              $(div).append($(cb));
              $(div).append($(space));
              $(div).append($(label));
              $(div).append($(element));

              if (flag.is_requesteeble) {
                var req = $('<input class="requestee"></input>');
                req.attr("name", 'flag_requestee-' + flag.id);
                req.attr("id", 'flag_requestee-' + flag.id);
                $(req).userautocomplete({
                  multiple: false
                });
                $(div).append($(space));
                $(div).append(req);
              }
              $(div).addClass('bz_default_hidden');

              flag_block.append($(div));
              selecterize(element);

            }

            product_block.append($(flag_block));
          }
        }

        var close_duplicate = $('<div class="field_block"><label for="set_dupe_of">Mark as Duplicate</label><input type="checkbox" name="set_dupe_of" value="1" id="set_dupe_of" onclick="markAsDupe(event);"> <input class="bz_default_hidden" type="text_input" name="dupe_of" value="" id="dupe_of"></div>');
        top_block.append($(close_duplicate));

        $(bug_form).append($(top_block));
        $(bug_form).append($(product_block));
        $(bug_form).append($(other_block));
        $(this.elements.content).append($(bug_form));
        $(this.elements.content).append(
          `
<hr>
<p>
  Beside most fields is a checkbox, if the field is set but the checkbox
  is unset, then changes will not be made to that field.
</p>
<p>
  If you change the value in a field then the checkbox will automatically check.
  You can manually uncheck the checkbox to prevent the change being made.
</p>
`
        );
      },
      prepare: function () {
        // this runs every time a dialog is shown
        if (first_show) {
          first_show = false;
          var pargs = {};
          if (Object.keys(selected_products).length == 1) {
            pargs['auto'] = 1;
            pargs['def'] = Object.keys(selected_products)[0];
            uncheck = true;
          } else {
            pargs['onclick'] = 1;
          }
          init_products(pargs);
          init_components({
            follow_product: 1
          });
          init_sub_components({
            myid: "rh_sub_components",
            hook_component: 1,
            nohide: 1,
            use_name: 1
          });
          $('#component').hide_field();
          $('#rh_sub_components').hide_field();

          $("#product").on('change', function () {
            if ($('#component').parent().hasClass('bz_default_hidden')) {
              $('#component').show_field();
              $('#rh_sub_components').show_field();
            }
            $('#component').fetch_all(true, true);
            if (uncheck) {
              uncheck = false;
              $('#changed_product').prop("checked", false);
            }
          });
        }
      },
      callback: function (closeEvent) {
        switch (closeEvent.index) {
          case 0:
            submit_bug_edits();
            break;
          case 1:
            // NOP
            break;
        }
      }
    };
  }, true);

  if (!alertify.editBugsBox) {
    alert("Check failed for editBugsBox!");
  }
}

function markAsDupe(e) {
  var $select = $("#status").selectize();
  var $selectize = $select[0].selectize;
  $selectize.setValue(BUGZILLA.param.duplicate_or_move_bug_status);

  $select = $("#resolution").selectize();
  $selectize = $select[0].selectize;
  $selectize.setValue('DUPLICATE');

  $('#dupe_of').removeClass('bz_default_hidden');
}

var bug_edit_dialog;

function bulk_edit_bugs(num_bugs, bug_list) {
  bug_ids = bug_list;
  if (bug_edit_dialog) {
    bug_edit_dialog.set('title', 'Bulk Edit ' + num_bugs + " bug[s]")
    bug_edit_dialog.resizeTo('80%', '70%').show();
  } else {
    bug_edit_dialog = alertify.editBugsBox(num_bugs).resizeTo('80%', '70%');
  }
}

function delete_job(ev, jobid) {
  var data = {};
  data['ids'] = [jobid];

  var message = 'This action will permanently and irrevocably remove the ob without processing it. Continue?';

  alertify.confirm(message, function (e) {
    if (e) {
      $(ev)._onAjaxSend();

      return rpc = new Rpc('RedHat', 'delete_jobs', data)
        .complete(function (cb) {
          $(ev)._onAjaxComplete();
        })
        .fail(function (error) {
          warn(error);
        })
        .done(function (res) {
          alertify.success("Job " + jobid + " deleted.");
          $('#job_row_' + jobid).remove();
        });
    }
  });
}

function update_classification() {
  var prod = $('#product').get_selected_item();
  var $cla = $('#classification').selectize()[0].selectize;
  var class_for_prod = all_classifications[prod];
  $cla.setValue(class_for_prod);
}

function show_hide_flags() {
  $('select.flag_select').each(function (index, field) {
    var indx = $(field).attr('data-index-number');
    var prod = $('#product').get_selected_item();
    var classification = $('#classification').get_selected_item();

    if (prod == "") return;
    if (BB_FIELDS['flagtypes.name'].values[indx].visibility_values.includes(prod) ||
      BB_FIELDS['flagtypes.name'].values[indx].visibility_values.includes(classification)) {
      $(this).parent().removeClass('bz_default_hidden');
    } else {
      $('#changed_flag_type-' + BB_FIELDS['flagtypes.name'].values[indx].id).prop("checked", false);
      $(this).parent().addClass('bz_default_hidden');
    }
  });
}

function view_job(ev, jobid) {
  var data = {};
  data['ids'] = [jobid];

  $(ev)._onAjaxSend();

  return rpc = new Rpc('RedHat', 'view_jobs', data)
    .complete(function (cb) {
      $(ev)._onAjaxComplete();
    })
    .fail(function (error) {
      warn(error);
    })
    .done(function (res) {
      alertify.alert('Job')
        .setting({
          'label': 'OK',
          'message': '<pre>' + JSON.stringify(res.jobs[0], null, 4) + '</pre>',
          'transition': 'fade'
        })
        .setHeader('Job ' + jobid)
        .show();
    });
}

function reschedule_job(ev, jobid) {
  var data = {};
  data['ids'] = [jobid];

  $(ev)._onAjaxSend();

  return rpc = new Rpc('RedHat', 'reschedule_jobs', data)
    .complete(function (cb) {
      $(ev)._onAjaxComplete();
    })
    .fail(function (error) {
      warn(error);
    })
    .done(function (res) {
      alertify.success("Job " + jobid + " rescheduled.");
      $('#job_row_' + jobid + ' .run_after').text(res.run_after);
    });
}

function ConfidentialDataWarning(noComment) {

  if (noComment === undefined) {
    noComment = false;
  }

  var msg = `<B>ATTENTION:</B> You are making this attachment and comment public, please ensure that the
  content does not contain any confidential data!
  `;

  if (noComment) {
    msg = `<B>ATTENTION:</B> You are making this attachment public, please ensure that the
    content does not contain any confidential data!
    `;
  }

  if (!$('#isprivate').prop("checked")) {
    var note = alertify.alert(
        'Confidential Data Warning',
        msg
      )
      .set({
        transition: 'pulse',
        pinnable: false,
        closable: false,
        modal: true
      });
  }
}

// 60K = 1 minute
var session_check_interval = 60000;

function checkSessionExpires() {
  var exp_time = $.cookie('Bugzilla_session_expires');

  if (typeof exp_time === 'undefined') return;

  var expTime = new Date(exp_time);
  var nowTime = new Date();

  var diff = expTime - nowTime;
  var seconds = Math.floor(diff / 1000);
  var minutes = Math.floor(seconds / 60);

  if (minutes > 0) {
    $("#session_time").html('Session expires at:<br>' + expTime.toLocaleString());
    $("#session_time").prop("title", "Your session will expire in " + minutes + " minutes.");
  } else {
    $("#session_time").html('Session expired at:<br>' + expTime.toLocaleString());
    $("#session_time").prop("title", "Your session has expired.");
  }

  if (minutes > 0 && minutes < 15) {
    $("#session_timer").html('<span class="minutes_remaining">' + minutes + '</span>');
    $("#session_timer span").prop("title", "Your session will expire in " + minutes + " minutes.");
    $("#session_timer").addClass('active');
    if (minutes <= 10) $("#session_timer span").addClass('amber');
    if (minutes <= 5) $("#session_timer span").addClass('red');
  } else if (minutes > 0) {
    $("#session_timer").html('');
    $("#session_timer").removeClass('active');
  } else {
    $("#session_timer").html('<span class="minutes_remaining red expired" title="Your session has expired. Use the re-login menu to start a new session.">Expired</span>');
  }

  setTimeout(checkSessionExpires, session_check_interval);
}

var group_col = [];
var group_col_index = {};
var data_hooks = [];

function resolve(path, obj) {
  return path.split('.').reduce(function (prev, curr) {
    return prev ? prev[curr] : null
  }, obj || self)
}

function update_data_hooks() {

  data_hooks = [];

  if (group_col[0]) {
    data_hooks[0] = function (row) {
      var fld_name = group_col[0];
      if (field_map[fld_name]) {
        fld_name = field_map[fld_name];
      }

      var val = resolve(fld_name, row);
      if (typeof (val) != 'undefined') {
        return format_field(val, row, fld_name, true);
      }

      return;
    };
  }

  if (group_col[1]) {
    data_hooks[1] = function (row) {
      var fld_name = group_col[1];
      if (field_map[fld_name]) {
        fld_name = field_map[fld_name];
      }

      var val = resolve(fld_name, row);
      if (typeof (val) != 'undefined') {
        return format_field(val, row, fld_name, true);
      }

      return;
    };
  }
}

function update_bug_list_fixed(table, level) {
  var fIndex = group_col[0];
  if (typeof fIndex === 'string') {
    //    fIndex = $.inArray(fIndex, the_search.include_fields);
    fIndex = $.inArray(fIndex, real_cols);
  }

  var order = table.order();
  var idx = $.map(order, function (item, i) {
    if (item[0] == fIndex)
      return i;
  })[0];

  var fixed_order = [];
  if (fIndex >= 0) {
    if (group_col_index[0] && group_col_index[0] != fIndex) {
      table.column(group_col_index[0]).visible(true);
    }
    group_col_index[0] = fIndex;

    var dir = 'asc';
    $.each(table.order(), function (i, data) {
      if (data[0] == fIndex) {
        dir = data[1];
      }
    });

    fixed_order[0] = [fIndex, dir];
    table.column(fIndex).visible(false);

    if (level == 1) {
      if (group_col[1]) {
        var sIndex = group_col[1];
        if (typeof sIndex === 'string') {
          sIndex = $.inArray(sIndex, the_search.include_fields);
        }

        if (sIndex >= 0) {
          if (group_col_index[1] && group_col_index[1] != sIndex && group_col_index[0] != group_col_index[1]) {
            table.column(group_col_index[1]).visible(true);
          }
          group_col_index[1] = sIndex;
          table.column(sIndex).visible(false);

          var sdir = 'asc';
          $.each(table.order(), function (i, data) {
            if (data[0] == fIndex) {
              sdir = data[1];
            }
          });

          fixed_order[1] = [sIndex, sdir];
        }
      }
    } else {
      if (group_col[1]) {
        var sIndex = group_col[1];
        if (typeof sIndex === 'string') {
          sIndex = $.inArray(sIndex, the_search.include_fields);
        }
        if (sIndex >= 0) {
          if (group_col_index[0] != sIndex) {
            table.column(sIndex).visible(true);
          }
        }
      }
    }
  }

  update_data_hooks();
  table.rowGroup().dataSrc(data_hooks);
  table.order.fixed({
    pre: fixed_order
  });

  table.draw();
}

function toggle_row_groups(collapse, most) {
  if (collapse) {
    $('.dtrg-level-1').addClass('group_collapsed');
    $('#bz_buglist tr').not('.bz_buglist_header, .dtrg-level-0, .dtrg-level-1').addClass('dtrg-level-1-hide');

    if (most && $('.dtrg-level-1').length >= 1) {
      $('.dtrg-level-0').removeClass('group_collapsed');

    } else {
      $('#bz_buglist tr').not('.bz_buglist_header, .dtrg-level-0').addClass('dtrg-level-0-hide');
      $('.dtrg-level-0').addClass('group_collapsed');
    }
  } else {
    $('.dtrg-level-0').removeClass('group_collapsed');
    $('.dtrg-level-1').removeClass('group_collapsed');
    $('.dtrg-level-0-hide').removeClass('dtrg-level-0-hide');
    $('.dtrg-level-1-hide').removeClass('dtrg-level-1-hide');
  }
}

function filter_rows(json) {
  // Clone array as we will be filtering it
  var fdata = json['data'];

  if (typeof fdata === 'undefined') return;

  if (typeof $('#bz_buglist').DataTable().settings()[0]._searchPanes !== 'undefined') {
    $.each($('#bz_buglist').DataTable().settings()[0]._searchPanes.s.panes, function () {
      if (!this.s.displayed) {
        return;
      }

      const selected = this.s.serverSelect;

      if (selected.length === 0) return;

      const field = the_search.include_fields[this.s.index];

      fdata = fdata.filter(function (row) {
        return selected.filter(sp => sp.display == row[field]).length > 0;
      });
    });
  }

  if (fdata.length != json['data'].length) {
    json['numRecordsFiltered'] = json['data'].length - fdata.length;
    json['data'] = fdata;
  } else {
    json['numRecordsFiltered'] = 0;
  }
}

function load_field_selector(current, target, extraPlugins, exclude) {
  var reqPlugins = ['remove_button', 'minimum_search_length', 'extra_keys_control', 'drag_drop', 'auto_position'];
  if (Array.isArray(extraPlugins)) $.merge(reqPlugins, extraPlugins);
  var selected = current.split(/,\s*/);

  $.each(selected, function (i, value) {
    var tmp = value.split(/\s+/);
    selected[i] = tmp[0];
    if (typeof (BB_FIELDS[selected[i]]) === 'undefined' &&
      typeof (ext_field_map[selected[i]]) !== 'undefined') {
      selected[i] = ext_field_map[selected[i]];
    }

    if (typeof (BB_FIELDS[selected[i]]) === 'undefined' &&
      typeof (field_map[selected[i]]) !== 'undefined') {
      selected[i] = field_map[selected[i]];
    }
  });

  var $select = $('<select id="' + target + '_select"></select>');
  $select.attr('multiple', true);

  $('#' + target).append($select);

  $select.selectize({
    sortField: {
      field: 'text',
      direction: 'asc'
    },
    plugins: reqPlugins,
    maxOptions: 'nolimit',
    minimum_search_length: 2,
    render: {
      option: function (item, escape) {
        var classes = "option";
        var extra = "";

        return '<div class="' + classes +
          '" title="' + escape(item.title) +
          '" id="' + escape(item.id) +
          '" data-value="' + escape(item.value) +
          '" data-name="' + escape(item.data_name) +
          '">' +
          '<span>' + escape(item.text) + '</span>' + extra +
          '</div>';
      }
    },
  });

  var $sel = $select.selectize()[0].selectize;

  $.each(BB_FIELDS, function (i, obj) {
    if (!obj['buglist']) return;
    if (Array.isArray(exclude) && (exclude.indexOf(obj['name']) > -1 || exclude.indexOf(obj['internal_name']) > -1)) {
      return;
    }

    var data = {};
    data['id'] = target + '_select_' + obj['name'];
    if (true) {
      data['data_name'] = obj['internal_name'];
      data['value'] = obj['name'];
      data['text'] = obj['display_name'] ? obj['display_name'] : obj['name'];
    } else {
      data['data_name'] = obj['name'];
      data['value'] = obj['internal_name'];
      data['text'] = obj['display_name'] ? obj['display_name'] : obj['internal_name'];
    }

    data['title'] = obj['long_desc'] ? obj['long_desc'] : '';
    if ($.inArray(data['value'], selected) >= 0 || $.inArray(data['data_name'], selected) >= 0) {
      data['selected'] = true;
    }
    $sel.addOption(data);
  });

  // Do this after adding options to preserve order
  $.each(selected, function (i, value) {
    $sel.addItem(value);
  });

  $sel.refreshOptions(false);
}

function hide_column(table, index) {
  $(table).DataTable().column(index).visible(false);
}


function complete_url(url) {
  var extra = '';

  var seen = [];

  // Row groups first
  var fixed_order = table.order.fixed()['pre'];
  $.each(fixed_order, function (i, data) {
    if (extra == '') extra = '&order=';
    extra = extra + real_cols[data[0]] + ' ' + data[1];
    seen.push(data[0]);

    if (i < fixed_order.length) {
      extra = extra + ', ';
    }
  });

  var order = table.order();

  // Then other sorting
  $.each(order, function (i, data) {
    if (seen.indexOf(data[0]) != -1) return;
    if (extra == '') extra = '&order=';
    extra = extra + real_cols[data[0]] + ' ' + data[1];
    seen.push(data[0]);

    if (i < order.length) {
      extra = extra + ', ';
    }
  });

  if (group_col[0]) {
    extra = extra + '&row_group1=1';
    if (group_col[1]) {
      extra = extra + '&row_group2=1';
    }
  }

  return url + extra;
}

function dt_submit() {
  if (typeof $.fn.dataTable == 'undefined') {
    return;
  }

  var tables = $.fn.dataTable.tables({
    visible: true,
    api: true
  });
  $.each(tables.context, function () {
    var $myform = this.nTable.closest('form');

    // Checked checkboxes
    $.each(this.oInstance.$('input:checked'), function () {
      add_hidden(this.name, this.value, $myform)
    });

    // Inputs in selected rows
    $.each(this.oInstance.$('tr.selected'), function () {
      $(this).find('input').each(function () {
        add_hidden(this.name, this.value, $myform)
      });
    });

    // select boxes
    $.each(this.oInstance.$('select'), function () {
      if (($(this).val() !== "") && ($(this).val() !== 0)) {
        add_hidden(this.name, this.value, $myform);
      }
    });
  });
}

function add_hidden(nm, val, $frm) {
  $('<input />').attr('type', 'hidden')
    .attr('name', nm)
    .attr('value', val)
    .appendTo($frm);
}

function invite_users() {
  var data = {};
  data['users'] = $('#users').val().split(/,\s*/);
  data['groups'] = $('#groups').get_selected_items();

  $("#commit")._onAjaxSend();

  return rpc = new Rpc('GroupInvite', 'invite_users', data)
    .complete(function (cb) {
      $('#commit')._onAjaxComplete();
    })
    .fail(function (error) {
      warn(error);
    })
    .done(function (res) {
      alertify.success("Update succeeded");
      $.each(res, function (login, data) {
        $("#results").append('<h3>' + login + '</h3>');
        if (typeof data.groups !== 'undefined') {
          $("#results").append('<p></p>Was added to these groups: ' + data.groups.join(', '));
        }
        if (typeof data.teams !== 'undefined') {
          $("#results").append('<p></p>Was added to these teams: ');
          $.each(data.teams, function (idx, team) {
            $("#results").append(team_link(team.name, team));
          });
        }
        if (typeof data.bug !== 'undefined') {
          $("#results").append('<p>Had this group request workflow bug openened: ' + bug_link(data.bug.id, data.bug));
        }
      })
    });
}

function format_component_search_link(data, row) {

  var url = `
  buglist.cgi?quicksearch=product="${row.product}"
  `;

  if (typeof row.component !== 'undefined') {
    url = url + `&component="${row.component}"&rh_sub_components="${data}"`;
  } else {
    url = url + `&component=${data}`;

  }

  var link = `
  <a href='${url}'>${row.label}</a>
  `;

  return link;
}

function format_component_search_link(data, row, role, field_name) {
  var label = row.label;
  var my_prod = escapeHtml(row.product);

  var url = `buglist.cgi?query_format=advanced&bug_status=__open__&product=${my_prod}`;
  var my_name = escapeHtml(row.name);
  if (typeof row.component !== 'undefined') {
    var comp_name = escapeHtml(row.component);
    url = url + `&component=${comp_name}&rh_sub_components=${my_name}`;
  } else {
    url = url + `&component=${my_name}`;
  }

  if (typeof role !== 'undefined' &&
    typeof row[role] !== 'undefined') {
    if (row[role] == '') {
      return '';
    }

    // Logged out users won't get email adresses
    if (typeof (BB_CONFIG.user.id) !== 'undefined') {
      url = url + `&f1=${field_name}&o1=equals&v1=${row[role]}`;
    }

    label = data;
  }

  var link = `<a class="nowrap" href='${url}'>${label}</a>`;

  return link;
}

if (!alertify.reportSPAMBox) {
  alertify.dialog('reportSPAMBox', function factory() {
    var notes = $('<p>Please enter a brief explanation of why you think this comment is SPAM.</p>');
    var input = $('<textarea rows="5" class="small"></textarea>');
    var space = document.createTextNode(" ");;

    return {
      main: function (login, bug_id, comment_id, comment_count) {
        this.login = login;
        this.bug_id = bug_id;
        this.comment_id = comment_id;
        this.comment_count = comment_count;
        this.set('title', 'Report comment ' + comment_count + ' as SPAM');
        this.set('pinnable', false);
        this.set('resizable', false);
        this.set('maximizable', false);
      },
      setup: function () {
        return {
          buttons: [{
              text: 'Submit',
              className: alertify.defaults.theme.ok,
            },
            {
              text: alertify.defaults.glossary.cancel,
              key: keys.ESC,
              invokeOnClose: true,
              className: alertify.defaults.theme.cancel,
            }
          ],
          focus: {
            element: null,
            select: false
          },
        };
      },
      build: function () {
        input.className = alertify.defaults.theme.input;
        this.elements.content.appendChild(notes[0]);
        this.elements.content.appendChild(space);
        this.elements.content.appendChild(input[0]);
      },
      prepare: function () {
        //$(input).val();
      },
      callback: function (closeEvent) {
        var returnValue;
        switch (closeEvent.index) {
          case 0:
            var new_val = input[0].value;

            if (new_val !== "") {
              $(input)._onAjaxSend();
              // RPC to update message
              var summary = "Report of Bug " + this.bug_id + '#c' + this.comment_count;
              var that = this;
              var rpc = new Rpc('Bug', 'create', {
                  product: "Bugzilla",
                  component: "Operations",
                  rh_sub_components: {
                    "Operations": ["SPAM or Abuse"]
                  },
                  short_desc: summary,
                  version: '5.0',
                  description: new_val,
                  groups: ['team Bugzilla']
                })
                .complete(function () {
                  $(input)._onAjaxComplete();
                })
                .fail(function (error) {
                  warn(error);
                })
                .done(function (res) {
                  var msg = "Your report has been filed as Bug <a href='show_bug.cgi?id=" + res.id + "'>" + res.id + "</a>";
                  alertify.success(msg, 15);
                  if (!$('#ct_' + that.comment_count).is(":visible")) {
                    $('#tag_toggle_' + that.comment_count).trigger('click');
                  }

                  YAHOO.bugzilla.commentTagging.add(that.comment_id, that.comment_count, ['spam']);
                });
            } else {
              closeEvent.cancel = true;
              alertify.warning("Please enter a brief explanation of why you think this comment is SPAM.");
              return;
            }
            break;
          case 1:
            // NOP
            break;
        }
        $(input).remove();
      }
    };
  }, true);
}

function report_spam(login, bug_id, comment_id, comment_count) {
  alertify.reportSPAMBox(login, bug_id, comment_id, comment_count).resizeTo('580px', '340px');
}
