#!/usr/bin/perl -T
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

use 5.10.1;
use strict;
use warnings;
use lib qw(. lib);

use Net::SAML2;
use MIME::Base64;

use Bugzilla;
use Bugzilla::Error;
use Bugzilla::Constants;

BEGIN { Bugzilla->extensions }
use Bugzilla::Extension::SAML2Auth::Util;
use Bugzilla::Extension::SAML2Auth::Login;

use constant VALIDATING => 1;

my $template      = Bugzilla->template;
my $cgi           = Bugzilla->cgi;
my $saml_response = $cgi->param('SAMLResponse');
$cgi->delete('SAMLResponse');

unless ($saml_response && $saml_response ne '') {
  ThrowUserError('saml_idp_response_missing');
}

my $post;
eval {
  $post
    = Net::SAML2::Binding::POST->new(
    {cacert => Bugzilla->params->{saml2auth_sp_cacert}});
};
ThrowUserError('saml_idp_fetch_error', {msg => $@}) if ($@);

my $xml = decode_base64($saml_response);

my $response;
eval { $response = $post->handle_response($saml_response); };
ThrowUserError('saml_idp_validation_error', {msg => $@}) if (VALIDATING && $@);

if ($response) {
  my $assertion;
  eval {
    $assertion = Net::SAML2::Protocol::Assertion->new_from_xml(xml => $xml);
  };

  ThrowUserError('saml_assertion_error', {msg => $@}) if ($@);

  Bugzilla->request_cache->{saml2auth_assertation} = $assertion;
  my $user = Bugzilla->login(LOGIN_REQUIRED);
}

print $cgi->redirect($cgi->param('RelayState') || 'index.cgi');

