# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

package Bugzilla::Extension::Workflows::GroupInvite;

use strict;
use warnings;
use 5.10.1;

use parent qw(Exporter);

our @EXPORT = qw(
  page_group_invite
);

use Bugzilla::Constants;
use Bugzilla::Error;
use Bugzilla::Util;
use Bugzilla::Extension::Workflows::Util;
use Bugzilla::Token;

sub page_group_invite {
  my ($vars) = @_;
  my $dbh    = Bugzilla->dbh;
  my $input  = Bugzilla->input_params;

  Bugzilla->login(LOGIN_REQUIRED);
  my $user = Bugzilla->user;

  my @groups;
  foreach my $group (Bugzilla::Group->get_all) {
    next                  if ($group->{category} eq 'Deprecated');
    push(@groups, $group) if (_can_invite($user, $group));
  }

  $vars->{'groups'} = \@groups;

  return;
}

sub _can_invite {
  my $user  = shift;
  my $group = shift;

  my $can  = 0;
  my $team = $group->my_team;
  my $gwf  = $group->group_workflow;

  if ($user->can_bless($group->id)) {
    $can = 1;
  }
  elsif ($gwf
    && $gwf->approval_group
    && $user->in_group_id($gwf->approval_group->id))
  {
    $can = 1;
  }
  elsif ($team && $team->user_can_edit()) {
    $can = 1;
  }

  return $can;
}

1;

__END__

=head1 NAME

Bugzilla::Extension::Workflows::GroupInvite

=head1 DESCRIPTION

This workflow allows users to invite other users to groups and teams.

=cut

